/*
 * dna_query.cpp
 *
 *  Created on: 2011/07/19
 *      Author: shu
 */

#include "dna_sequence.h"
#include "aligner_parameters.h"
#include "sequence.h"
#include "dna_query.h"
#include "sa_sequence_index.h"
#include <string>
#include <stdexcept>

using namespace std;

DnaQuery::DnaQuery(const Sequence &sequence, AlignerParameters &parameters) :
Query(parameters), parameters_(&parameters), encoder_(*(parameters.GetAlignedType())), dna_length_(sequence.GetSequenceData().size()), is_set_statical_parameters_(false)
{
  SetName(sequence);
  SetEncodedSequence(sequence, encoder_, parameters);
}

void DnaQuery::SetName(const Sequence &sequence) {
  name_ = sequence.GetName();
}

void DnaQuery::SetEncodedSequence(const Sequence &sequence, const Encoder &encoder, const AlignerParameters &parameters) {
  string c_sequence = DnaSequence::GetComplementaryStrand(sequence.GetSequenceData());
  sequence_.resize(DnaQuery::GetSequenceLength(sequence.GetSequenceData().size()));
  sequence_[0] = AlignerParameters::kSequenceEnd;
  encoder.Encode(sequence.GetSequenceData(), &sequence_[1]);
  sequence_[1 + sequence.GetSequenceData().size() ] = AlignerParameters::kSequenceEnd;
  encoder.Encode(c_sequence, &sequence_[2 + sequence.GetSequenceData().size()]);
  sequence_[ 2 + 2*sequence.GetSequenceData().size() ] = AlignerParameters::kSequenceEnd;
}

void DnaQuery::SetIndex(const vector<Code> &sequence, const Encoder &encoder) {
  index_ = new SaSequenceIndex(&sequence[0], sequence.size(), encoder);
}
void DnaQuery::SetStaticalParameters(const vector<Code> &sequence, const AlignerParameters &parameters) {
  Statistics statistics(*(parameters.GetAlignedType()));
  try {
    statistics.CalculateUngappedKarlinParameters(&sequence[0], sequence.size(),
              parameters.GetScoreMatrix(), &ungapped_karlin_parameters_);
  }catch (invalid_argument e) {
    ungapped_karlin_parameters_ = parameters.GetUnGappedIdealKarlinParameters();
  }
  ungapped_extend_nominal_cutoff_ = Statistics::Normalized2Nominal(parameters.GetUngappedExtendNormalizedCutoff(), ungapped_karlin_parameters_);
  if (ungapped_extend_nominal_cutoff_ > parameters.GetUngappedIdealExtendNominalCutoff()) {
    ungapped_extend_nominal_cutoff_ = parameters.GetUngappedIdealExtendNominalCutoff();
  }
  gapped_extend_nominal_cutoff_ = parameters.GetGappedIdealExtendNominalCutoff();
  gapped_extend_nominal_trigger_ = Statistics::Normalized2Nominal(parameters.GetGappedExtendNormalizedTrigger(), ungapped_karlin_parameters_);
  if (gapped_extend_nominal_trigger_ > parameters.GetGappedIdealExtendNominalTrigger()) {
    gapped_extend_nominal_trigger_ = parameters.GetGappedIdealExtendNominalTrigger();
  }
  is_set_statical_parameters_ = true;

}
